@@ -141,7 +141,8 @@ extern "C" {
 
 /* Useful but undefined in perlapi */
 #define FLOATSIZE sizeof(float)
-#define BOOLSIZE sizeof(bool) // ha!
+#define BOOLSIZE sizeof(bool)      // ha!
+#define XDOUBLESIZE sizeof(double) // ugh...
 
 const char *file = __FILE__;
 
@@ -157,10 +158,10 @@ START_MY_CXT
 
 typedef struct CoW {
     DCCallback *cb;
-    struct CoW *next;
+    // struct CoW *next;
 } CoW;
 
-static CoW *cow;
+// static CoW *cow;
 
 typedef struct {
     char *sig;
@@ -185,6 +186,8 @@ typedef struct {
     dTHXfield(perl)
 } Callback;
 
+SV *ptr2sv(pTHX_ DCpointer ptr, SV *type);
+
 // http://www.catb.org/esr/structure-packing/#_structure_alignment_and_padding
 /* Returns the amount of padding needed after `offset` to ensure that the
 following address will be aligned to `alignment`. */
@@ -297,102 +300,124 @@ SV *enum2sv(pTHX_ SV *type, int in) {
 }
 
 char cbHandler(DCCallback *cb, DCArgs *args, DCValue *result, DCpointer userdata) {
-    // warn("here at %s line %d", __FILE__, __LINE__);
     Callback *cbx = (Callback *)userdata;
-    // warn("Triggering callback: %c (%s [%d] return: %c) at %s line %d", cbx->ret, cbx->sig,
-    // cbx->sig_len, cbx->ret, __FILE__, __LINE__);
+    //~ warn("Triggering callback: %s [%d] return: %c at %s line %d", cbx->sig, cbx->sig_len,
+    //~ cbx->ret, __FILE__, __LINE__);
     dTHXa(cbx->perl);
-
     dSP;
     int count;
 
     ENTER;
     SAVETMPS;
-
     PUSHMARK(SP);
 
-    EXTEND(SP, cbx->sig_len);
-    char type;
-    for (int i = 0; i < cbx->sig_len; ++i) {
-        type = cbx->sig[i];
-        // warn("type : %c", type);
-        switch (type) {
-        case DC_SIGCHAR_VOID:
-            // TODO: push undef?
-            break;
-        case DC_SIGCHAR_BOOL:
-            mPUSHs(boolSV(dcbArgBool(args)));
-            break;
-        case DC_SIGCHAR_CHAR:
-            mPUSHi((IV)dcbArgChar(args));
-            break;
-        case DC_SIGCHAR_UCHAR:
-            mPUSHu((UV)dcbArgChar(args));
-            break;
-        case DC_SIGCHAR_SHORT:
-            mPUSHi((IV)dcbArgShort(args));
-            break;
-        case DC_SIGCHAR_USHORT:
-            mPUSHu((UV)dcbArgShort(args));
-            break;
-        case DC_SIGCHAR_INT:
-            mPUSHi((IV)dcbArgInt(args));
-            break;
-        case DC_SIGCHAR_UINT:
-            mPUSHu((UV)dcbArgInt(args));
-            break;
-        case DC_SIGCHAR_LONG:
-            mPUSHi((IV)dcbArgLong(args));
-            break;
-        case DC_SIGCHAR_ULONG:
-            mPUSHu((UV)dcbArgLong(args));
-            break;
-        case DC_SIGCHAR_LONGLONG:
-            mPUSHi((IV)dcbArgLongLong(args));
-            break;
-        case DC_SIGCHAR_ULONGLONG:
-            mPUSHu((UV)dcbArgLongLong(args));
-            break;
-        case DC_SIGCHAR_FLOAT:
-            mPUSHn((NV)dcbArgFloat(args));
-            break;
-        case DC_SIGCHAR_DOUBLE:
-            mPUSHn((NV)dcbArgDouble(args));
-            break;
-        case DC_SIGCHAR_POINTER: {
-            mPUSHs(sv_setref_pv(newSV(1), "Affix::Pointer", dcbArgPointer(args)));
-        } break;
-        case DC_SIGCHAR_STRING: {
-            DCpointer ptr = dcbArgPointer(args);
-            PUSHs(newSVpv((char *)ptr, 0));
-        } break;
-        case DC_SIGCHAR_BLESSED: {
-            DCpointer ptr = dcbArgPointer(args);
-            HV *blessed = MUTABLE_HV(SvRV(*av_fetch(cbx->args, i, 0)));
-            SV **package = hv_fetchs(blessed, "package", 0);
-            PUSHs(sv_setref_pv(newSV(1), SvPV_nolen(*package), ptr));
-        } break;
-        case DC_SIGCHAR_ENUM:
-        case DC_SIGCHAR_ENUM_UINT: {
-            PUSHs(enum2sv(aTHX_ * av_fetch(cbx->args, i, 0), dcbArgInt(args)));
-        } break;
-        case DC_SIGCHAR_ENUM_CHAR: {
-            PUSHs(enum2sv(aTHX_ * av_fetch(cbx->args, i, 0), dcbArgChar(args)));
-        } break;
-        case DC_SIGCHAR_ANY: {
-            DCpointer ptr = dcbArgPointer(args);
-            SV *sv = newSV(0);
-            if (ptr != NULL && SvOK(MUTABLE_SV(ptr))) { sv = MUTABLE_SV(ptr); }
-            PUSHs(sv);
-        } break;
-        default:
-            croak("Unhandled callback arg. Type: %c [%s]", cbx->sig[i], cbx->sig);
-            break;
-        }
-    }
-    PUTBACK;
+    if (cbx->sig_len) {
+        EXTEND(SP, cbx->sig_len);
+        char type;
+        for (size_t i = 0; i < cbx->sig_len; ++i) {
+            type = cbx->sig[i];
+            //~ warn("type %d: %c", i, type);
+            //~ {
+            //~ SV *_type = SvRV(*av_fetch(cbx->args, i, 0));
+            //~ sv_dump(_type);
+            //~ }
+            switch (type) {
+            case DC_SIGCHAR_VOID:
+                // TODO: push undef?
+                break;
+            case DC_SIGCHAR_BOOL:
+                mPUSHs(boolSV(dcbArgBool(args)));
+                break;
+            case DC_SIGCHAR_CHAR:
+                mPUSHi((IV)dcbArgChar(args));
+                break;
+            case DC_SIGCHAR_UCHAR:
+                mPUSHu((UV)dcbArgChar(args));
+                break;
+            case DC_SIGCHAR_SHORT:
+                mPUSHi((IV)dcbArgShort(args));
+                break;
+            case DC_SIGCHAR_USHORT:
+                mPUSHu((UV)dcbArgShort(args));
+                break;
+            case DC_SIGCHAR_INT:
+                mPUSHi((IV)dcbArgInt(args));
+                break;
+            case DC_SIGCHAR_UINT:
+                mPUSHu((UV)dcbArgInt(args));
+                break;
+            case DC_SIGCHAR_LONG:
+                mPUSHi((IV)dcbArgLong(args));
+                break;
+            case DC_SIGCHAR_ULONG:
+                mPUSHu((UV)dcbArgLong(args));
+                break;
+            case DC_SIGCHAR_LONGLONG:
+                mPUSHi((IV)dcbArgLongLong(args));
+                break;
+            case DC_SIGCHAR_ULONGLONG:
+                mPUSHu((UV)dcbArgLongLong(args));
+                break;
+            case DC_SIGCHAR_FLOAT:
+                mPUSHn((NV)dcbArgFloat(args));
+                break;
+            case DC_SIGCHAR_DOUBLE:
+                mPUSHn((NV)dcbArgDouble(args));
+                break;
+            case DC_SIGCHAR_POINTER: {
+                DCpointer ptr = dcbArgPointer(args);
+                SV *__type = *av_fetch(cbx->args, i, 0);
+                char *_type = SvPV_nolen(__type);
+                // warn("_type == %s", _type);
+                switch (_type[0]) { // true type
+                case DC_SIGCHAR_ANY: {
+                    SV *s = ptr2sv(aTHX_ ptr, (__type));
+                    sv_dump(s);
+                    mPUSHs(s);
+                } break;
+                case DC_SIGCHAR_CODE: {
+                    Callback *cb = (Callback *)dcbGetUserData((DCCallback *)ptr);
+                    mPUSHs(cb->cv);
+                } break;
+                default:
+                    mPUSHs(sv_setref_pv(newSV(1), "Affix::Pointer", ptr));
+                    break;
+                }
+            } break;
+            case DC_SIGCHAR_STRING: {
+                DCpointer ptr = dcbArgPointer(args);
+                PUSHs(newSVpv((char *)ptr, 0));
+            } break;
+            case DC_SIGCHAR_BLESSED: {
+                DCpointer ptr = dcbArgPointer(args);
+                HV *blessed = MUTABLE_HV(SvRV(*av_fetch(cbx->args, i, 0)));
+                SV **package = hv_fetchs(blessed, "package", 0);
+                PUSHs(sv_setref_pv(newSV(1), SvPV_nolen(*package), ptr));
+            } break;
+            case DC_SIGCHAR_ENUM:
+            case DC_SIGCHAR_ENUM_UINT: {
+                PUSHs(enum2sv(aTHX_ * av_fetch(cbx->args, i, 0), dcbArgInt(args)));
+            } break;
+            case DC_SIGCHAR_ENUM_CHAR: {
+                PUSHs(enum2sv(aTHX_ * av_fetch(cbx->args, i, 0), dcbArgChar(args)));
+            } break;
+            case DC_SIGCHAR_ANY: {
+                DCpointer ptr = dcbArgPointer(args);
+                SV *sv = newSV(0);
+                if (ptr != NULL && SvOK(MUTABLE_SV(ptr))) { sv = MUTABLE_SV(ptr); }
+                PUSHs(sv);
+            } break;
+            default:
+                croak("Unhandled callback arg. Type: %c [%s]", cbx->sig[i], cbx->sig);
+                break;
+            }
+        }  }
+        PUTBACK;
+
+    warn("here at %s line %d", __FILE__, __LINE__);
     if (cbx->ret == DC_SIGCHAR_VOID) { call_sv(cbx->cv, G_VOID); }
     else {
+        warn("here at %s line %d", __FILE__, __LINE__);
         count = call_sv(cbx->cv, G_SCALAR);
         if (count != 1) croak("Big trouble: %d returned items", count);
         SPAGAIN;
@@ -523,7 +548,7 @@ static size_t _sizeof(pTHX_ SV *type) {
     case DC_SIGCHAR_FLOAT:
         return FLOATSIZE;
     case DC_SIGCHAR_DOUBLE:
-        return DOUBLESIZE;
+        return XDOUBLESIZE;
     case DC_SIGCHAR_STRUCT:
     case DC_SIGCHAR_UNION:
     case DC_SIGCHAR_ARRAY:
@@ -548,11 +573,11 @@ static size_t _offsetof(pTHX_ SV *type) {
 }
 
 static DCaggr *_aggregate(pTHX_ SV *type) {
-    // warn("here at %s line %d", __FILE__, __LINE__);
-    //  //sv_dump(type);
+    //~ warn("_aggregate( ... ) at %s line %d", __FILE__, __LINE__);
+    //~ sv_dump(type);
 
     char *str = SvPVbytex_nolen(type); // stringify to sigchar; speed cheat vs sv_derived_from(...)
-                                       // warn("here at %s line %d", __FILE__, __LINE__);
+    // warn("here at %s line %d", __FILE__, __LINE__);
     size_t size = _sizeof(aTHX_ type);
 
     switch (str[0]) {
@@ -605,10 +630,15 @@ static DCaggr *_aggregate(pTHX_ SV *type) {
                        array_len,
                          __FILE__, __LINE__);*/
                 } break;
+                case DC_SIGCHAR_ANY:
+                case DC_SIGCHAR_CODE:
+                case DC_SIGCHAR_POINTER: {
+                    dcAggrField(agg, DC_SIGCHAR_POINTER, offset, 1);
+                } break;
                 default: {
+                    //~ warn("dcAggrField(agg, %c, %d, 1); at %s line %d", str[0], offset, __FILE__,
+                    //~ __LINE__);
                     dcAggrField(agg, str[0], offset, 1);
-                    // warn("dcAggrField(agg, %c, %d, 1); at %s line %d", str[0], offset, __FILE__,
-                    //     __LINE__);
                 } break;
                 }
             }
@@ -636,8 +666,8 @@ static DCaggr *_aggregate(pTHX_ SV *type) {
 SV *ptr2sv(pTHX_ DCpointer ptr, SV *type) {
     SV *RETVAL = newSV(0);
     char *_type = SvPV_nolen(type);
-    // sv_dump(type);
-    //  warn("ptr2sv( %p, '%s' ); at %s line %d", ptr, _type, __FILE__, __LINE__);
+    //~ sv_dump(type);
+    //~ warn("ptr2sv( %p, '%s' ); at %s line %d", ptr, _type, __FILE__, __LINE__);
 
     switch (_type[0]) {
     case DC_SIGCHAR_VOID:
@@ -747,7 +777,6 @@ SV *ptr2sv(pTHX_ DCpointer ptr, SV *type) {
 
         SvSetSV(RETVAL, cb->cv);
         // warn("here at %s line %d", __FILE__, __LINE__);
-
     } break;
     default:
         croak("Oh, this is unexpected: %c", _type[0]);
@@ -935,62 +964,47 @@ void sv2ptr(pTHX_ SV *type, SV *data, DCpointer ptr, bool packed) {
     case DC_SIGCHAR_CODE: {
         // warn("here at %s line %d", __FILE__, __LINE__);
         DCCallback *cb = NULL;
+
+        // warn("here at %s line %d", __FILE__, __LINE__);
+        HV *field = MUTABLE_HV(SvRV(type)); // Make broad assumptions
+        SV **sig = hv_fetchs(field, "signature", 0);
+        SV **sig_len = hv_fetchs(field, "sig_len", 0);
+        SV **ret = hv_fetchs(field, "return", 0);
+        SV **args = hv_fetchs(field, "args", 0);
+
+        Callback *callback;
+        Newxz(callback, 1, Callback);
+
+        callback->args = MUTABLE_AV(SvRV(*args));
+        callback->sig = SvPV_nolen(*sig);
+        callback->sig_len = SvIV(*sig_len);
+        callback->ret = (char)*SvPV_nolen(*ret);
+
+        callback->cv = SvREFCNT_inc(data);
+        storeTHX(callback->perl);
+        // warn("here at %s line %d", __FILE__, __LINE__);
+        cb = dcbNewCallback(callback->sig, cbHandler, callback);
+        // warn("here at %s line %d", __FILE__, __LINE__);
         {
-            CoW *p = cow;
-            while (p != NULL) {
-                if (p->cb) {
-                    Callback *_cb = (Callback *)dcbGetUserData(p->cb);
-                    if (SvRV(_cb->cv) == SvRV(data)) {
-                        cb = p->cb;
-                        break;
-                    }
-                }
-                p = p->next;
-            }
+            CoW *hold;
+            Newxz(hold, 1, CoW);
+            // warn("here at %s line %d", __FILE__, __LINE__);
+            hold->cb = cb;
+            // warn("here at %s line %d", __FILE__, __LINE__);
+            Copy(hold, ptr, 1, DCpointer);
+            // warn("here at %s line %d", __FILE__, __LINE__);
         }
         // warn("here at %s line %d", __FILE__, __LINE__);
-        if (!cb) {
-            HV *field = MUTABLE_HV(SvRV(type)); // Make broad assumptions
-            SV **sig = hv_fetchs(field, "signature", 0);
-            SV **sig_len = hv_fetchs(field, "sig_len", 0);
-            SV **ret = hv_fetchs(field, "return", 0);
-            SV **args = hv_fetchs(field, "args", 0);
 
-            Callback *callback;
-            Newxz(callback, 1, Callback);
-
-            callback->args = MUTABLE_AV(SvRV(*args));
-            callback->sig = SvPV_nolen(*sig);
-            callback->sig_len = strlen(callback->sig);
-            callback->ret = (char)*SvPV_nolen(*ret);
-
-            callback->cv = SvREFCNT_inc(data);
-            storeTHX(callback->perl);
-            // warn("here at %s line %d", __FILE__, __LINE__);
-            cb = dcbNewCallback(callback->sig, cbHandler, callback);
-            // warn("here at %s line %d", __FILE__, __LINE__);
-            {
-                CoW *hold;
-                Newxz(hold, 1, CoW);
-                // warn("here at %s line %d", __FILE__, __LINE__);
-                hold->cb = cb;
-                hold->next = cow;
-                cow = hold;
-                // warn("here at %s line %d", __FILE__, __LINE__);
-                Copy(hold, ptr, 1, DCpointer);
-                // warn("here at %s line %d", __FILE__, __LINE__);
-            }
-            // warn("here at %s line %d", __FILE__, __LINE__);
-        }
         // warn("here at %s line %d", __FILE__, __LINE__);
     } break;
     default: {
         char *str = SvPVbytex_nolen(type);
         croak("%c is not a known type in sv2ptr(...)", str[0]);
     }
-        warn("here at %s line %d", __FILE__, __LINE__);
+        //~ warn("here at %s line %d", __FILE__, __LINE__);
     }
-    warn("here at %s line %d", __FILE__, __LINE__);
+    //~ warn("here at %s line %d", __FILE__, __LINE__);
 
     return;
 }
